﻿using System;
using System.Collections;
using Alachisoft.NCache.Runtime.Events;
using Alachisoft.NCache.Web.Caching;
using Alachisoft.NCache.sample.data;

// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Continuous Query sample
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================


namespace ContinuousQuerySample
{
	public class ContinuousQuerySample
	{
		public static void Main(string[] args)
		{
			try
			{
				//Initialize new cache
				Cache cache;
				cache = NCache.InitializeCache("mypartitionedcache");
				cache.Clear();

				//New customer product 
				Customer customer = new Customer("John Mathew", 40, "25564-4546", "92-A Barnes Road, WA, Ct.", "Male");

				cache.Add("Customer:JohnMathew", customer);

                string query = "SELECT Alachisoft.NCache.sample.data.Customer WHERE this.Gender = ?";

				Hashtable values = new Hashtable();
				values["Gender"] = "Male";

				ContinuousQuery continuousQuery = new ContinuousQuery(query, values);

				//Add query notifications
                
				//Item added notification

                 continuousQuery.RegisterNotification(new QueryDataNotificationCallback(QueryDataModified), EventType.ItemAdded, EventDataFilter.None);
              
				//Item update notification
                
                continuousQuery.RegisterNotification(new QueryDataNotificationCallback(QueryDataModified), EventType.ItemUpdated, EventDataFilter.None);

				//Item delete notification

                continuousQuery.RegisterNotification(new QueryDataNotificationCallback(QueryDataModified), EventType.ItemRemoved, EventDataFilter.None);

               
				//Register continuous query on server
				cache.RegisterCQ(continuousQuery);

				//getting keys via query
				ICollection keys = cache.SearchCQ(continuousQuery);
				if (keys.Count > 0)
				{
					IEnumerator itor = keys.GetEnumerator();
					while (itor.MoveNext())
					{
						string key = (string) itor.Current;
						Customer customerFound = (Customer) cache.Get(key);
						PrintCustomerDetails(customerFound);
					}
				}

				//getting keyvalue pair via query
				//Much faster way ... avoids round trip to cache
				IDictionary result = cache.SearchEntriesCQ(continuousQuery);
				if (result.Count > 0)
				{
					IEnumerator iter = result.Keys.GetEnumerator();
					while (iter.MoveNext())
					{
						Customer customerFound = (Customer) result[iter.Current];
						PrintCustomerDetails(customerFound);
					}
				}

				customer.Age = 35;

				//This should invoke query RegisterUpdateNotification 
				cache.Insert("Customer:JohnMathew", customer);

				Console.WriteLine();

                cache.UnRegisterCQ(continuousQuery);

                //Must dispose cache
				cache.Dispose();

				Environment.Exit(0);

			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
				Environment.Exit(0);
			}
		}

        private static void QueryDataModified(string @string, CQEventArg cqea)
        {
            switch (cqea.EventType)
            {
                case EventType.ItemAdded: Console.WriteLine(@string);
                    break;

                case EventType.ItemRemoved: Console.WriteLine(@string);
                    break;

                case EventType.ItemUpdated: Console.WriteLine(@string);
                    break;

                default:
                    break;
            }
        }

       private static void PrintCustomerDetails(Customer customer)
		 {
			Console.WriteLine();
			Console.WriteLine("Customer Details are as follows: ");
			Console.WriteLine("Name:       " + customer.Name);
			Console.WriteLine("Age:        " + customer.Age);
			Console.WriteLine("Gender:     " + customer.Gender);
			Console.WriteLine("Contact No: " + customer.ContactNo);
			Console.WriteLine("Address:    " + customer.Address);
		 }
	}

}